@@ -38,7 +38,8 @@ from samba.dsdb import (
     DS_DOMAIN_FUNCTION_2003,
     DS_DOMAIN_FUNCTION_2008_R2,
     DS_DOMAIN_FUNCTION_2012_R2,
-    DS_DOMAIN_FUNCTION_2016
+    DS_DOMAIN_FUNCTION_2016,
+    DS_GUID_USERS_CONTAINER
 )
 from samba.descriptor import (
     get_domain_descriptor,
@@ -69,12 +70,22 @@ def get_domainguid(samdb, domaindn):
 
 
 def get_dnsadmins_sid(samdb, domaindn):
-    res = samdb.search(base="CN=DnsAdmins,CN=Users,%s" % domaindn, scope=ldb.SCOPE_BASE,
-                       attrs=["objectSid"])
+    base_dn = "CN=DnsAdmins,%s" % samdb.get_wellknown_dn(ldb.Dn(samdb,
+                                            domaindn), DS_GUID_USERS_CONTAINER)
+    res = samdb.search(base=base_dn, scope=ldb.SCOPE_BASE, attrs=["objectSid"])
     dnsadmins_sid = ndr_unpack(security.dom_sid, res[0]["objectSid"][0])
     return dnsadmins_sid
 
 
+# Note: these classes are not quite the same as similar looking ones
+# in ../dnsserver.py -- those ones are based on
+# dnsserver.DNS_RPC_RECORD ([MS-DNSP]2.2.2.2.5 "DNS_RPC_RECORD"),
+# these are based on dnsp.DnssrvRpcRecord ([MS-DNSP] 2.3.2.2
+# "DnsRecord").
+#
+# They are not interchangeable or mergeable. If you're talking over
+# the wire you want those other ones; these are the on-disk format.
+
 class ARecord(dnsp.DnssrvRpcRecord):
 
     def __init__(self, ip_addr, serial=1, ttl=900, rank=dnsp.DNS_RANK_ZONE):
@@ -97,10 +108,10 @@ class AAAARecord(dnsp.DnssrvRpcRecord):
         self.data = ip6_addr
 
 
-class CNameRecord(dnsp.DnssrvRpcRecord):
+class CNAMERecord(dnsp.DnssrvRpcRecord):
 
     def __init__(self, cname, serial=1, ttl=900, rank=dnsp.DNS_RANK_ZONE):
-        super(CNameRecord, self).__init__()
+        super().__init__()
         self.wType = dnsp.DNS_TYPE_CNAME
         self.rank = rank
         self.dwSerial = serial
@@ -256,7 +267,6 @@ def setup_dns_partitions(samdb, domainsid, domaindn, forestdn, configdn,
             "SECDESC": b64encode(descriptor).decode('utf8')
         })
 
-    domainzone_guid = get_domainguid(samdb, domainzone_dn)
     domainzone_guid = str(uuid.uuid4())
     domainzone_dns = ldb.Dn(samdb, domainzone_dn).canonical_ex_str().strip()
 
@@ -278,7 +288,6 @@ def setup_dns_partitions(samdb, domainsid, domaindn, forestdn, configdn,
     })
 
     if fill_level != FILL_SUBDOMAIN:
-        forestzone_guid = get_domainguid(samdb, forestzone_dn)
         forestzone_guid = str(uuid.uuid4())
         forestzone_dns = ldb.Dn(samdb, forestzone_dn).canonical_ex_str().strip()
 
@@ -447,7 +456,7 @@ def add_ns_glue_record(samdb, container_dn, prefix, host):
 
 
 def add_cname_record(samdb, container_dn, prefix, host):
-    cname_record = CNameRecord(host)
+    cname_record = CNAMERecord(host)
     msg = ldb.Message(ldb.Dn(samdb, "%s,%s" % (prefix, container_dn)))
     msg["objectClass"] = ["top", "dnsNode"]
     msg["dnsRecord"] = ldb.MessageElement(ndr_pack(cname_record), ldb.FLAG_MOD_ADD, "dnsRecord")
@@ -675,7 +684,7 @@ def secretsdb_setup_dns(secretsdb, names, private_dir, binddns_dir, realm,
         key_version_number = 1
 
     # This will create the dns.keytab file in the private_dir when it is
-    # commited!
+    # committed!
     setup_ldb(secretsdb, setup_path("secrets_dns.ldif"), {
             "REALM": realm,
             "DNSDOMAIN": dnsdomain,
@@ -689,7 +698,7 @@ def secretsdb_setup_dns(secretsdb, names, private_dir, binddns_dir, realm,
 
 
 def create_dns_dir(logger, paths):
-    """Write out a DNS zone file, from the info in the current database.
+    """(Re)create the DNS directory and chown it to bind.
 
     :param logger: Logger object
     :param paths: paths object
@@ -714,6 +723,48 @@ def create_dns_dir(logger, paths):
                     dns_dir, paths.bind_gid))
 
 
+def create_dns_dir_keytab_link(logger, paths):
+    """Create link for BIND to DNS keytab
+
+    :param logger: Logger object
+    :param paths: paths object
+    """
+    private_dns_keytab_path = os.path.join(paths.private_dir, paths.dns_keytab)
+    bind_dns_keytab_path = os.path.join(paths.binddns_dir, paths.dns_keytab)
+
+    if os.path.isfile(private_dns_keytab_path):
+        if os.path.isfile(bind_dns_keytab_path):
+            try:
+                os.unlink(bind_dns_keytab_path)
+            except OSError as e:
+                logger.error("Failed to remove %s: %s" %
+                             (bind_dns_keytab_path, e.strerror))
+
+        # link the dns.keytab to the bind-dns directory
+        try:
+            os.link(private_dns_keytab_path, bind_dns_keytab_path)
+        except OSError as e:
+            logger.error("Failed to create link %s -> %s: %s" %
+                         (private_dns_keytab_path, bind_dns_keytab_path, e.strerror))
+
+        # chown the dns.keytab in the bind-dns directory
+        if paths.bind_gid is not None:
+            try:
+                os.chmod(paths.binddns_dir, 0o770)
+                os.chown(paths.binddns_dir, -1, paths.bind_gid)
+            except OSError:
+                if 'SAMBA_SELFTEST' not in os.environ:
+                    logger.info("Failed to chown %s to bind gid %u",
+                                paths.binddns_dir, paths.bind_gid)
+            try:
+                os.chmod(bind_dns_keytab_path, 0o640)
+                os.chown(bind_dns_keytab_path, -1, paths.bind_gid)
+            except OSError:
+                if 'SAMBA_SELFTEST' not in os.environ:
+                    logger.info("Failed to chown %s to bind gid %u",
+                                bind_dns_keytab_path, paths.bind_gid)
+
+
 def create_zone_file(lp, logger, paths, targetdir, dnsdomain,
                      hostip, hostip6, hostname, realm, domainguid,
                      ntdsguid, site):
@@ -933,7 +984,7 @@ def create_named_conf(paths, realm, dnsdomain, dns_backend, logger):
     """
 
     # TODO: This really should have been done as a top level import.
-    # It is done here to avoid a depencency loop.  That is, we move
+    # It is done here to avoid a dependency loop.  That is, we move
     # ProvisioningError to another file, and have all the provision
     # scripts import it from there.
 
@@ -964,6 +1015,7 @@ def create_named_conf(paths, realm, dnsdomain, dns_backend, logger):
         bind9_12 = '#'
         bind9_14 = '#'
         bind9_16 = '#'
+        bind9_18 = '#'
         if bind_info.upper().find('BIND 9.8') != -1:
             bind9_8 = ''
         elif bind_info.upper().find('BIND 9.9') != -1:
@@ -978,6 +1030,8 @@ def create_named_conf(paths, realm, dnsdomain, dns_backend, logger):
             bind9_14 = ''
         elif bind_info.upper().find('BIND 9.16') != -1:
             bind9_16 = ''
+        elif bind_info.upper().find('BIND 9.18') != -1:
+            bind9_18 = ''
         elif bind_info.upper().find('BIND 9.7') != -1:
             raise ProvisioningError("DLZ option incompatible with BIND 9.7.")
         elif bind_info.upper().find('BIND_9.13') != -1:
@@ -997,7 +1051,8 @@ def create_named_conf(paths, realm, dnsdomain, dns_backend, logger):
                     "BIND9_11": bind9_11,
                     "BIND9_12": bind9_12,
                     "BIND9_14": bind9_14,
-                    "BIND9_16": bind9_16
+                    "BIND9_16": bind9_16,
+                    "BIND9_18": bind9_18
                     })
 
 
@@ -1252,6 +1307,7 @@ def setup_bind9_dns(samdb, secretsdb, names, paths, lp, logger,
                         key_version_number=key_version_number)
 
     create_dns_dir(logger, paths)
+    create_dns_dir_keytab_link(logger, paths)
 
     if dns_backend == "BIND9_FLATFILE":
         create_zone_file(lp, logger, paths, targetdir, site=site,
