@@ -37,6 +37,7 @@ import samba.dcerpc.base
 from random import randint
 from random import SystemRandom
 from contextlib import contextmanager
+import shutil
 import string
 try:
     from samba.samdb import SamDB
@@ -50,11 +51,8 @@ import samba.ndr
 import samba.dcerpc.dcerpc
 import samba.dcerpc.epmapper
 
-try:
-    from unittest import SkipTest
-except ImportError:
-    class SkipTest(Exception):
-        """Test skipped."""
+from unittest import SkipTest
+
 
 BINDIR = os.path.abspath(os.path.join(os.path.dirname(__file__),
                                       "../../../../bin"))
@@ -259,8 +257,10 @@ class LdbTestCase(TestCase):
         self.filename = self.tempfile.name
         self.ldb = samba.Ldb(self.filename)
 
-    def set_modules(self, modules=[]):
+    def set_modules(self, modules=None):
         """Change the modules for this Ldb."""
+        if modules is None:
+            modules = []
         m = ldb.Message()
         m.dn = ldb.Dn(self.ldb, "@MODULES")
         m["@LIST"] = ",".join(modules)
@@ -295,6 +295,40 @@ class TestCaseInTempDir(TestCase):
                 print("could not remove temporary file: %s" % e,
                       file=sys.stderr)
 
+    def rm_files(self, *files, allow_missing=False, _rm=os.remove):
+        """Remove listed files from the temp directory.
+
+        The files must be true files in the directory itself, not in
+        sub-directories.
+
+        By default a non-existent file will cause a test failure (or
+        error if used outside a test in e.g. tearDown), but if
+        allow_missing is true, the absence will be ignored.
+        """
+        for f in files:
+            path = os.path.join(self.tempdir, f)
+
+            # os.path.join will happily step out of the tempdir,
+            # so let's just check.
+            if os.path.dirname(path) != self.tempdir:
+                raise ValueError("{path} might be outside {self.tempdir}")
+
+            try:
+                _rm(path)
+            except FileNotFoundError as e:
+                if not allow_missing:
+                    raise AssertionError(f"{f} not in {self.tempdir}: {e}")
+
+                print(f"{f} not in {self.tempdir}")
+
+    def rm_dirs(self, *dirs, allow_missing=False):
+        """Remove listed directories from temp directory.
+
+        This works like rm_files, but only removes directories,
+        including their contents.
+        """
+        self.rm_files(*dirs, allow_missing=allow_missing, _rm=shutil.rmtree)
+
 
 def env_loadparm():
     lp = param.LoadParm()
@@ -308,7 +342,7 @@ def env_loadparm():
 def env_get_var_value(var_name, allow_missing=False):
     """Returns value for variable in os.environ
 
-    Function throws AssertionError if variable is defined.
+    Function throws AssertionError if variable is undefined.
     Unit-test based python tests require certain input params
     to be set in environment, otherwise they can't be run
     """
@@ -326,18 +360,6 @@ class RpcInterfaceTestCase(TestCase):
     """DCE/RPC Test case."""
 
 
-class ValidNetbiosNameTests(TestCase):
-
-    def test_valid(self):
-        self.assertTrue(samba.valid_netbios_name("FOO"))
-
-    def test_too_long(self):
-        self.assertFalse(samba.valid_netbios_name("FOO" * 10))
-
-    def test_invalid_characters(self):
-        self.assertFalse(samba.valid_netbios_name("*BLA"))
-
-
 class BlackboxProcessError(Exception):
     """This is raised when check_output() process returns a non-zero exit status
 
@@ -371,7 +393,8 @@ class BlackboxProcessError(Exception):
 class BlackboxTestCase(TestCaseInTempDir):
     """Base test case for blackbox tests."""
 
-    def _make_cmdline(self, line):
+    @staticmethod
+    def _make_cmdline(line):
         """Expand the called script into a fully resolved path in the bin
         directory."""
         if isinstance(line, list):
@@ -436,8 +459,9 @@ class BlackboxTestCase(TestCaseInTempDir):
     # where ret is the return code
     #       stdout is a string containing the commands stdout
     #       stderr is a string containing the commands stderr
-    def run_command(self, line):
-        line = self._make_cmdline(line)
+    @classmethod
+    def run_command(cls, line):
+        line = cls._make_cmdline(line)
         use_shell = not isinstance(line, list)
         p = subprocess.Popen(line,
                              stdout=subprocess.PIPE,
